#include "aquarium.h"
#include "draw.h"

#include "background.h"

static Background_settings background_settings;
static AquariumData *ad;

Background_settings *background_get_settings_ptr(void)
{
    return &background_settings;
}

static void background_prepare_solid(unsigned char r, unsigned char g, unsigned b)
{
    int i,j,t;

    for (i = 0; i < ad->ymax; i++) {
	for (j = 0; j < ad->xmax; j++) {
	    t = (i * ad->xmax * 3) + j * 3;
	    ad->bgr[t + 0] = r;
	    ad->bgr[t + 1] = g;
	    ad->bgr[t + 2] = b;
	}
    }

}

static void background_prepare_shaded(unsigned char r1, unsigned char g1, unsigned b1,
				      unsigned char r2, unsigned char g2, unsigned b2)
{
    int i,j,t;
    float d1,d2,d3;

    d1 = ((float)r2 - (float)r1) / (float) ad->ymax;
    d2 = ((float)g2 - (float)g1) / (float) ad->ymax;
    d3 = ((float)b2 - (float)b1) / (float) ad->ymax;

    for (i = 0; i < ad->ymax; i++) {
	for (j = 0; j < ad->xmax; j++) {
	    t = (i * ad->xmax * 3) + j * 3;
	    ad->bgr[t + 0] = (unsigned char)((float)r1 + ((float) i * d1));
	    ad->bgr[t + 1] = (unsigned char)((float)g1 + ((float) i * d2));
	    ad->bgr[t + 2] = (unsigned char)((float)b1 + ((float) i * d3));
	}
    }

}



static void background_prepare_image(char *image_file, int user_choise)
{
    GdkPixbuf *water=NULL, *water2=NULL;
    GError *imerr=NULL;
    unsigned char *buff;
    int i, j, k=0, y, rowstride, alpha;

    if(user_choise){
	water = gdk_pixbuf_new_from_file(image_file, &imerr);
	if (water==NULL){
	    gai_display_error_continue("Can't load image!\nChanging to default water image background.");
	    water = gai_load_image("water.png");
	}
    } else {
	water = gai_load_image("water.png");
    }
    
    water2 = gdk_pixbuf_scale_simple(water, ad->xmax, ad->ymax,
				     GDK_INTERP_BILINEAR);
    buff = gdk_pixbuf_get_pixels(water2);

    rowstride = gdk_pixbuf_get_rowstride(water2);
    alpha = (int)gdk_pixbuf_get_has_alpha(water2);
    
    for (i = 0; i < ad->ymax; i++) {
	y = i * rowstride;
	for (j = 0; j < ((ad->xmax * (3+alpha))); j += (3+alpha)) {
	    ad->bgr[k + 0] = buff[y + j + 0];
	    ad->bgr[k + 1] = buff[y + j + 1];
	    ad->bgr[k + 2] = buff[y + j + 2];
	    k += 3;
	}
    }

    g_object_unref(water);
    g_object_unref(water2);

}


void background_exit(void)
{

}

void background_init(void)
{
    ad = aquarium_get_settings_ptr();

    if(background_settings.solid)
	background_prepare_solid(background_settings.r1,
				 background_settings.g1,
				 background_settings.b1);

    if(background_settings.shaded)
	background_prepare_shaded(background_settings.r2,
				  background_settings.g2,
				  background_settings.b2,
				  background_settings.r3,
				  background_settings.g3,
				  background_settings.b3);

    if(background_settings.image)
	background_prepare_image(background_settings.imagename, 1);

    if(background_settings.wateralike)
	background_prepare_image(NULL,0);

}
