
/*

  Preference window for Sherman's aquarium, 
  using the General Applet Interface.

*/

#include <stdio.h>
#include <gai/gai.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#include "clock.h"
#include "date.h"
#include "background.h"
#include "soundeffects.h"
#include "bottom.h"
#include "bubble.h"
#include "fish.h"
#include "over.h"
#include "leds.h"
#include "thermometer.h"
#include "xmms_sn.h"
#include "settings.h"
#include "status.h"

#define DATE_DRAW_OM 0
#define DATE_VERT_OM 1
#define DATE_HORZ_OM 2
#define CLOCK_VERT_OM 3
#define CLOCK_HORZ_OM 4
#define CLOCK_DRAW_OM 5
#define CLOCK_TYPE_OM 6
#define CLOCK_FONTSIZE_OM 7
#define LEDS1_COLOUR_OM 8
#define LEDS2_COLOUR_OM 9
#define LEDS3_COLOUR_OM 10
#define LEDS1_FUNC_OM 11
#define LEDS2_FUNC_OM 12
#define LEDS3_FUNC_OM 13
#define LEDS_VERT_OM 14
#define LEDS_HORZ_OM 15
#define LEDS_DRAW_OM 16
#define LEDS_VERT_HORZ_OM 17
#define LEDS_LAYOUT_OM 18
#define LEDS4_FUNC_OM 19
#define LEDS4_COLOUR_OM 20
#define XMMS_DRAW_OM 21
#define XMMS_VERT_OM 22
#define XMMS_HORZ_OM 23
#define XMMS_SCROLL_OM 24

#define THERMO_DRAW1_OM 25
#define THERMO_VERT1_OM 26
#define THERMO_HORZ1_OM 27

#define THERMO_DRAW2_OM 28
#define THERMO_VERT2_OM 29
#define THERMO_HORZ2_OM 30
#define THERMO_MESSURE1_OM 31
#define THERMO_MESSURE1_S_OM 32
#define THERMO_MESSURE2_OM 33
#define THERMO_MESSURE2_S_OM 34


#define SOUND_RADIO 3
#define CLOCK_VERT_RADIO 4
#define CLOCK_HORZ_RADIO 5
#define CLOCK_DRAW_RADIO 6
#define CLOCK_VAL_RADIO 7
#define CLOCK_DIGITAL_FONT_RADIO 8
#define BACKGROUND_RADIO 9

#define FISH_POP_RADIO 11
#define OVER_RADIO 12
#define FISH_POP_F_RADIO 13
#define LED1_USE_RADIO 14
#define LED2_USE_RADIO 15
#define LED3_USE_RADIO 16
#define LED1_COLOUR_RADIO 17
#define LED2_COLOUR_RADIO 18
#define LED3_COLOUR_RADIO 19
#define LEDS_DRAW_RADIO 20
#define LEDS_VERT_RADIO 21
#define LEDS_HORZ_RADIO 22
#define LEDS_LAYOUT_RADIO 23 

#define DATE_FONT_COLOUR 0
#define CLOCK_HOUR_COLOUR 1
#define CLOCK_MIN_COLOUR 2
#define CLOCK_SEC_COLOUR 3
#define CLOCK_DIGITAL_FONT_COLOUR 4
#define CLOCK_FUZZY_COLOUR 5
#define BG_SOLID_COLOUR 6
#define BG_SHADED1_COLOUR 7
#define BG_SHADED2_COLOUR 8
#define THERMO1_COLOUR 9
#define THERMO2_COLOUR 10
#define THERMO1_S_COLOUR 11
#define THERMO2_S_COLOUR 12

#define XMMS_FONT_COLOUR 13


#define A_BG_FS 1

static Date_settings date_settings;
static Clock_settings clock_settings;
static Background_settings background_settings;
static Sound_settings sound_settings;
static Bottom_settings bottom_settings;
static Bubble_settings bubble_settings;
static Fish_settings fish_settings;
static Over_settings over_settings;
static Leds_settings leds_settings;
static Thermometer_settings thermometer_settings;
static Xmms_sn_settings xmms_settings;
static General_settings general_settings;

static char *draw_list[] = {"Background", "Forground",NULL};
static char *vert_list[] = {"Top", "Center", "Bottom", NULL};
static char *horz_list[] = {"Left", "Center", "Right", NULL};


/* Static needed variables */
static int const_0 = 0;
static int const_1 = 1;
static int const_5 = 5;
static int const_10 = 10;
static int const_20 = 20;
static int const_50 = 50;
static int const_100 = 100;
static int const_200 = 200;
static int const_1000 = 1000;
static int const_100000000 = 100000000;

/*static char *sound_prg;
static char *sound_prg_new;
*/


/*--------------------------------------------------------------------------------*/
/* DATE */
/*--------------------------------------------------------------------------------*/
static GaiOptionMenu date_draw_om = {"Draw on:", 
				     draw_list, 
				     &date_settings.draw, 
				     &date_settings.draw,
				     DATE_DRAW_OM};

static GaiOptionMenu date_vert_om = {"Vertical placement:",
				     vert_list, 
				     &date_settings.vert,
				     &date_settings.vert,
				     DATE_VERT_OM};

static GaiOptionMenu date_horz_om = {"Horizontal placement:",
				     horz_list, 
				     &date_settings.horz,
				     &date_settings.horz,
				     DATE_HORZ_OM};

static GaiCheckButton date_button = {"On", 
				     &date_settings.on,
				     &date_settings.on};
static GaiColourSelector date_colour = {"Font colour",
					&date_settings.r,
					&date_settings.g,
					&date_settings.b,
					&date_settings.alpha,
					DATE_FONT_COLOUR};

static GaiBox date_box[] = {{GAI_CHECKBUTTON, 		&date_button},
			    {GAI_OPTIONMENU,		&date_draw_om},
			    {GAI_OPTIONMENU,		&date_vert_om},
			    {GAI_OPTIONMENU,		&date_horz_om},
			    {GAI_COLOURSELECTOR, 	&date_colour},
			    {GAI_END, 			NULL}};

static GaiFrame date_frame = {"Date",NULL,(void *)&date_box,NULL};

/*--------------------------------------------------------------------------------*/
/* Clock */
/*--------------------------------------------------------------------------------*/

char *clock_type_list[] = {"Off","Analog","Digital","Fuzzy",NULL};
char *clock_fontsize_list[] = {"Large", "Small", NULL};

static GaiOptionMenu clock_vert_om = {"Vertical placement:",
				      vert_list,
				      &clock_settings.vert,
				      &clock_settings.vert,
				      CLOCK_VERT_OM};

static GaiOptionMenu clock_horz_om = {"Horizontal placement:",
				      horz_list,
				      &clock_settings.horz,
				      &clock_settings.horz,
				      CLOCK_HORZ_OM};
static GaiOptionMenu clock_draw_om = {"Draw on:", 
				      draw_list,
				      &clock_settings.draw,
				      &clock_settings.draw,
				      CLOCK_DRAW_OM};

static GaiOptionMenu clock_type_om = {"Clock type:", clock_type_list,
				      &clock_settings.type,
				      &clock_settings.type,
				      CLOCK_TYPE_OM};

static GaiOptionMenu clock_fontsize_om = {"Font size:", clock_fontsize_list,
					  &clock_settings.digital_fontsize,
					  &clock_settings.digital_fontsize,
					  CLOCK_FONTSIZE_OM};


static GaiCheckButton analog_keep_circular = {"Keep the clock circular", 
					      &clock_settings.analog_keep_circular, 
					      &clock_settings.analog_keep_circular};

static GaiColourSelector analog_hour_pointer = {"Colour of the hour pointer",
						&clock_settings.analog_colour_hour_r,
						&clock_settings.analog_colour_hour_g,
						&clock_settings.analog_colour_hour_b,
						&clock_settings.analog_colour_hour_alpha,
						CLOCK_HOUR_COLOUR};

static GaiColourSelector analog_min_pointer = {"Colour of the minute pointer",
					       &clock_settings.analog_colour_min_r,
					       &clock_settings.analog_colour_min_g,
					       &clock_settings.analog_colour_min_b,
					       &clock_settings.analog_colour_min_alpha,
					       CLOCK_MIN_COLOUR};

static GaiColourSelector analog_sec_pointer = {"Colour of the second pointer",
					       &clock_settings.analog_colour_sec_r,
					       &clock_settings.analog_colour_sec_g,
					       &clock_settings.analog_colour_sec_b,
					       &clock_settings.analog_colour_sec_alpha,
					       CLOCK_SEC_COLOUR};

static GaiCheckButton analog_with_seconds_button = {"With seconds", 
						    &clock_settings.analog_seconds, 
						    &clock_settings.analog_seconds};

static GaiBox analog_box[] = {{GAI_CHECKBUTTON, 	&analog_keep_circular},
			      {GAI_CHECKBUTTON, 	&analog_with_seconds_button},
			      {GAI_COLOURSELECTOR, 	&analog_hour_pointer},
			      {GAI_COLOURSELECTOR, 	&analog_min_pointer},
			      {GAI_COLOURSELECTOR, 	&analog_sec_pointer},
			      {GAI_END, 		NULL}};

static GaiCheckButton digital_blink_button = {"Make the colon blink", 
					      &clock_settings.digital_blinking, 
					      &clock_settings.digital_blinking};
static GaiColourSelector digital_font_colour_sel = {"Font colour",
						    &clock_settings.digital_colour_r,
						    &clock_settings.digital_colour_g,
						    &clock_settings.digital_colour_b,
						    &clock_settings.digital_colour_alpha,
						    CLOCK_DIGITAL_FONT_COLOUR};
static GaiCheckButton digital_with_seconds_button = {"With seconds", 
						     &clock_settings.digital_seconds, 
						     &clock_settings.digital_seconds};


static GaiBox digital_box[] = {{GAI_CHECKBUTTON, 	&digital_blink_button},
			       {GAI_CHECKBUTTON, 	&digital_with_seconds_button},
			       {GAI_COLOURSELECTOR, 	&digital_font_colour_sel},
			       {GAI_OPTIONMENU, 	&clock_fontsize_om},
			       {GAI_END, 		NULL}};



static GaiFrame analog_digital_frame = {"Analog", 
					"Digital", 
					(void *)&analog_box, 
					(void *)&digital_box};


static GaiColourSelector fuzzy_font_colour_sel = {"Font colour",
						  &clock_settings.fuzzy_colour_r,
						  &clock_settings.fuzzy_colour_g,
						  &clock_settings.fuzzy_colour_b,
						  &clock_settings.fuzzy_colour_alpha,
						  CLOCK_FUZZY_COLOUR};


static GaiBox fuzzy_box[] = {{GAI_COLOURSELECTOR, 	&fuzzy_font_colour_sel},
			     {GAI_END,			NULL}};

static GaiFrame fuzzy_frame = {"Fuzzy", 
			       NULL,
			       (void *)&fuzzy_box,
			       NULL};

static GaiBox clock_box[] = {{GAI_OPTIONMENU,	&clock_type_om},
			     {GAI_OPTIONMENU,	&clock_draw_om},
			     {GAI_OPTIONMENU,	&clock_vert_om},
			     {GAI_OPTIONMENU,	&clock_horz_om},
			     {GAI_FRAME,	&analog_digital_frame},
			     {GAI_FRAME,	&fuzzy_frame},
			     {GAI_END,		NULL}};


static GaiFrame clock_frame = {"Clock",NULL,	(void *)&clock_box, NULL};

static GaiBox clock_date_box[] = {{GAI_FRAME, 	&date_frame},
				  {GAI_FRAME, 	&clock_frame},
				  {GAI_END,	NULL}};
				  

/*--------------------------------------------------------------------------------*/
/* Background settings */
/*--------------------------------------------------------------------------------*/

static GaiRadioButton bg_type_radio1 = {"Solid colour", 
					&background_settings.solid,
					&background_settings.solid,
					BACKGROUND_RADIO};

static GaiRadioButton bg_type_radio2 = {"Shaded colour", 
					&background_settings.shaded,
					&background_settings.shaded,
					BACKGROUND_RADIO};

static GaiRadioButton bg_type_radio3 = {"Wateralike", 
					&background_settings.wateralike,
					&background_settings.wateralike,
					BACKGROUND_RADIO};

static GaiRadioButton bg_type_radio4 = {"Image", 
					&background_settings.image,
					&background_settings.image,
					BACKGROUND_RADIO};


static GaiBox bg_type_box[] = {{GAI_RADIOBUTTON, &bg_type_radio1},
			       {GAI_RADIOBUTTON, &bg_type_radio2},
			       {GAI_RADIOBUTTON, &bg_type_radio3},
			       {GAI_RADIOBUTTON, &bg_type_radio4},
			       {GAI_END, 	NULL}};

static GaiFrame bg_type = {"Type", NULL, (void *)&bg_type_box, NULL};


static GaiColourSelector bg_solid_colour_sel = {"Solid colour",
						&background_settings.r1,
						&background_settings.g1,
						&background_settings.b1,
						&background_settings.alpha1,
						BG_SOLID_COLOUR};

static GaiColourSelector bg_shaded_colour1_sel = {"Shaded - Top colour",
						  &background_settings.r2,
						  &background_settings.g2,
						  &background_settings.b2,
						  &background_settings.alpha2,
						  BG_SHADED1_COLOUR};

static GaiColourSelector bg_shaded_colour2_sel = {"Shaded - Bottom colour",
						  &background_settings.r3,
						  &background_settings.g3,
						  &background_settings.b3,
						  &background_settings.alpha3,
						  BG_SHADED2_COLOUR};


static GaiFileSelector bg_file_sel = {"Background image", 
				      (void *)&background_settings.imagename,
				      (void *)&background_settings.imagename_new,
				      A_BG_FS};

static GaiBox bg_box[] = {{GAI_FRAME, 		&bg_type},
			  {GAI_COLOURSELECTOR, 	&bg_solid_colour_sel},
			  {GAI_COLOURSELECTOR, 	&bg_shaded_colour1_sel},
			  {GAI_COLOURSELECTOR, 	&bg_shaded_colour2_sel},
			  {GAI_FILESELECTOR, 	&bg_file_sel},
			  {GAI_END, 		NULL}};

/*--------------------------------------------------------------------------------*/
/* General */
/*--------------------------------------------------------------------------------*/

static GaiCheckButton sound_on_butt = {"Sound effects", 
				       &sound_settings.on,
				       &sound_settings.on};

static GaiTextEntry sound_cmd = {"External sound playing program: ",
				 (void *)&sound_settings.prg,
				 (void *)&sound_settings.prg};

static GaiRadioButton sound_ogg_mp3_radio1 = {"Use Mp3 files for sound effects", 
					      &sound_settings.type_mp3,
					      &sound_settings.type_mp3,
					      SOUND_RADIO};
static GaiRadioButton sound_ogg_mp3_radio2 = {"Use Ogg files for sound effects", 
					      &sound_settings.type_ogg,
					      &sound_settings.type_ogg,
					      SOUND_RADIO};


static GaiBox sound_ogg_mp3_box[] = {{GAI_CHECKBUTTON,	&sound_on_butt},
				     {GAI_TEXTENTRY, 	&sound_cmd},
				     {GAI_RADIOBUTTON, 	&sound_ogg_mp3_radio1},
				     {GAI_RADIOBUTTON, 	&sound_ogg_mp3_radio2},
				     {GAI_END, 		NULL}};


static GaiSpinButton max_bubbles = {"Max bubbles",
				    &bubble_settings.max_bubbles,
				    &const_0,
				    &const_100,
				    &const_1,
				    &bubble_settings.max_bubbles};

static GaiCheckButton have_bottom = {"Have sea floor", 
				     &bottom_settings.have_sea_floor,
				     &bottom_settings.have_sea_floor};

static GaiCheckButton random_plants = {"Random number of plants up to the number below",
				       &bottom_settings.random_plants,
				       &bottom_settings.random_plants};
static GaiSpinButton num_plants = {"Number of plants:", 
				   &bottom_settings.max_plants,
				   &const_1,
				   &const_100,
				   &const_1,
				   &bottom_settings.max_plants};
static GaiSpinButton scale_plants = {"Scale of plants:",
				     &bottom_settings.scale,
				     &const_10,
				     &const_1000,
				     &const_5,
				     &bottom_settings.scale};

static GaiSpinButton bottom_animals = {"Number of bottom animals:",
				       &bottom_settings.num_bottom_animals,
				       &const_0,
				       &const_50,
				       &const_1,
				       &bottom_settings.num_bottom_animals};



static GaiBox bottom_box[] = {{GAI_CHECKBUTTON,  &have_bottom},
			      {GAI_CHECKBUTTON,  &random_plants},
			      {GAI_SPINBUTTON,   &num_plants},
			      {GAI_SPINBUTTON,   &scale_plants},
			      {GAI_SPINBUTTON,	&bottom_animals},
			      {GAI_END,		NULL}};


static GaiSpinButton ratio_width = {"Width:",
				    &general_settings.ratio_width,
				    &const_1,
				    &const_20,
				    &const_1,
				    &general_settings.ratio_width};
static GaiSpinButton ratio_height = {"Height:",
				    &general_settings.ratio_height,
				    &const_1,
				    &const_20,
				    &const_1,
				    &general_settings.ratio_height};
static GaiText ratio_text = {"The ratios might be compromised\n"
			     "in order to fit the gnome panel.\n"
			     "When the gnome panel is in vertical\n"
			     "mode, the ratios will be rotated 90\n"
			     "degrees."};

static GaiBox ratio_box[] = {{GAI_SPINBUTTON, &ratio_width},
			     {GAI_SPINBUTTON, &ratio_height},
			     {GAI_TEXT,       &ratio_text},
			     {GAI_END,	      NULL}};

static GaiFrame bottom_frame = {"Bottom", "Background", (void *)&bottom_box, (void *)&bg_box};


static GaiFrame sound_n_ratio = {"Sound format",
				 "Applet size ratio",
				 (void *)&sound_ogg_mp3_box,
				 (void *)&ratio_box};



static GaiBox box[] = {{GAI_FRAME, 	&sound_n_ratio},
		       {GAI_SPINBUTTON, &max_bubbles},
		       {GAI_FRAME, 	&bottom_frame},
		       {GAI_END, 	NULL}};

/*--------------------------------------------------------------------------------*/
/* Fish */
/*--------------------------------------------------------------------------------*/


static GaiSpinButton fish1_spin = {"Fish 1:",
				   &fish_settings.fish1,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.fish1};
static GaiSpinButton fish2_spin = {"Fish 2:",
				   &fish_settings.fish2,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.fish2};
static GaiSpinButton fish3_spin = {"Fish 3:",
				   &fish_settings.fish3,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.fish3};
static GaiSpinButton fish4_spin = {"Fish 4:",
				   &fish_settings.fish4,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.fish4};
static GaiSpinButton fish5_spin = {"Fish 5:",
				   &fish_settings.fish5,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.fish5};
static GaiSpinButton fish6_spin = {"Fish 6:",
				   &fish_settings.fish6,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.fish6};
static GaiSpinButton swordfish_spin = {"Swordfish:",
				       &fish_settings.swordfish,
				       &const_0,
				       &const_100,
				       &const_1,
				       &fish_settings.swordfish};

static GaiSpinButton blowfish_spin = {"Blowfish:",
				      &fish_settings.blowfish,
				      &const_0,
				      &const_100,
				      &const_1,
				      &fish_settings.blowfish};

static GaiSpinButton fillmore_spin = {"Fillmore:",
				      &fish_settings.fillmore,
				      &const_0,
				      &const_100,
				      &const_1,
				      &fish_settings.fillmore};

static GaiSpinButton sherman_spin = {"Sherman:",
				     &fish_settings.sherman,
				     &const_0,
				     &const_100,
				     &const_1,
				     &fish_settings.sherman};

static GaiSpinButton megan_spin = {"Megan:",
				   &fish_settings.megan,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.megan};

static GaiSpinButton prey_spin = {"Prey:",
				  &fish_settings.prey,
				  &const_0,
				  &const_100,
				  &const_1,
				  &fish_settings.prey};

static GaiSpinButton hunter_spin = {"Hunter:",
				    &fish_settings.hunter,
				    &const_0,
				    &const_100,
				    &const_1,
				    &fish_settings.hunter};

static GaiSpinButton lori_spin = {"Lori:",
				  &fish_settings.lori,
				  &const_0,
				  &const_100,
				  &const_1,
				  &fish_settings.lori};
static GaiSpinButton ernest_spin = {"Ernest:",
				    &fish_settings.ernest,
				    &const_0,
				    &const_100,
				    &const_1,
				    &fish_settings.ernest};
static GaiSpinButton squid_spin = {"Squid:",
				   &fish_settings.squid,
				   &const_0,
				   &const_100,
				   &const_1,
				   &fish_settings.squid};

static GaiSpinButton bdweller_spin = {"Bottomdweller:",
				      &fish_settings.bdweller,
				      &const_0,
				      &const_100,
				      &const_1,
				      &fish_settings.bdweller};
static GaiSpinButton hawthorne_spin = {"Hawthorne:",
				       &fish_settings.hawthorne,
				       &const_0,
				       &const_100,
				       &const_1,
				       &fish_settings.hawthorne};



static GaiRadioButton select_fish = {"User defined selection and population", 
				     &fish_settings.selection_fish,
				     &fish_settings.selection_fish,
				     FISH_POP_RADIO};
static GaiRadioButton random_fish = {"Random selection and given population size", 
				     &fish_settings.random_fish,
				     &fish_settings.random_fish,
				     FISH_POP_RADIO};
static GaiRadioButton random_pop_fish = {"Random selection and random population size", 
					 &fish_settings.random_pop_fish,
					 &fish_settings.random_pop_fish,
					 FISH_POP_RADIO};

static GaiSpinButton num_fish = {"Population size:",
				 &fish_settings.num_fish,
				 &const_0,
				 &const_200,
				 &const_1,
				 &fish_settings.num_fish};

static GaiBox num_fish_box[] = {{GAI_RADIOBUTTON, 	&select_fish},
				{GAI_RADIOBUTTON, 	&random_fish},
				{GAI_SPINBUTTON,	&num_fish},
				{GAI_RADIOBUTTON,	&random_pop_fish},
				{GAI_END,		NULL}};



static GaiCheckButton eat_fish = {"Eating prey fish is ok",
				  &fish_settings.eat,
				  &fish_settings.eat};

static GaiCheckButton explode_fish = {"Make blowfish explode is ok",
				      &fish_settings.explode,
				      &fish_settings.explode};
static GaiCheckButton rebirth_fish = {"Dead fish might be reborn",
				      &fish_settings.rebirth,
				      &fish_settings.rebirth};
static GaiCheckButton scale_diff_fish = {"Fish from the same kind may differ in size.",
					 &fish_settings.scale_diff,
					 &fish_settings.scale_diff};


static GaiSpinButton scale_fish = {"Scale (in %):",
				   &fish_settings.scale,
				   &const_10,
				   &const_1000,
				   &const_5,
				   &fish_settings.scale};
static GaiSpinButton speed_fish = {"Speed (in %):",
				   &fish_settings.speed,
				   &const_10,
				   &const_1000,
				   &const_5,
				   &fish_settings.speed};

static GaiSpinButton hunter_fish = {"Hunter agressiveness (in %):",
				    &fish_settings.hunter_agr,
				    &const_0,
				    &const_100,
				    &const_5,
				    &fish_settings.hunter_agr};
static GaiSpinButton swordfish_fish = {"Swordfish agressiveness (in %):",
				       &fish_settings.swordfish_agr,
				       &const_0,
				       &const_100,
				       &const_5,
				       &fish_settings.swordfish_agr};




static GaiBox fish_box[] = {{GAI_CHECKBUTTON,	&eat_fish},
			    {GAI_CHECKBUTTON,	&explode_fish},
			    {GAI_CHECKBUTTON,	&rebirth_fish},
			    {GAI_CHECKBUTTON,	&scale_diff_fish},
			    {GAI_SPINBUTTON,	&scale_fish},
			    {GAI_SPINBUTTON,	&speed_fish},
			    {GAI_SPINBUTTON,	&hunter_fish},
			    {GAI_SPINBUTTON,	&swordfish_fish},
			    {GAI_END,		NULL}};

static GaiFrame num_fish_frame = {"Fish population", 
				  "Fish behavior",
				  (void *)&num_fish_box, 
				  (void *)&fish_box};


static GaiBox fish_box_left[] = {{GAI_SPINBUTTON,	&fish1_spin},
				 {GAI_SPINBUTTON,	&fish2_spin},
				 {GAI_SPINBUTTON,	&fish3_spin},
				 {GAI_SPINBUTTON,	&fish4_spin},
				 {GAI_SPINBUTTON,	&fish5_spin},
				 {GAI_SPINBUTTON,	&fish6_spin},
				 {GAI_SPINBUTTON,	&swordfish_spin},
				 {GAI_SPINBUTTON,	&blowfish_spin},
				 {GAI_SPINBUTTON,	&bdweller_spin},
				 {GAI_END,		NULL}};

static GaiBox fish_box_right[] = {{GAI_SPINBUTTON,	&fillmore_spin},
				  {GAI_SPINBUTTON,	&sherman_spin},
				  {GAI_SPINBUTTON,	&megan_spin},
				  {GAI_SPINBUTTON,	&prey_spin},
				  {GAI_SPINBUTTON,	&hunter_spin},
				  {GAI_SPINBUTTON,	&lori_spin},
				  {GAI_SPINBUTTON,	&ernest_spin},
				  {GAI_SPINBUTTON,	&squid_spin},
				  {GAI_SPINBUTTON,	&hawthorne_spin},
				  {GAI_END,		NULL}};

static GaiFrame f_fish_frame = {"Fish", 
				"More Fish",
				(void *)&fish_box_left,
				(void *)&fish_box_right};

static GaiBox main_fish_box[] = {{GAI_FRAME, 	&f_fish_frame},
				 {GAI_FRAME, 	&num_fish_frame},
				 {GAI_END,	NULL}};

/*--------------------------------------------------------------------------------*/
/* Extra */
/*--------------------------------------------------------------------------------*/
static char *xmms_scroll_list[] = {"Vertical","Horizontal",NULL};

static GaiCheckButton over_cursor_off = {"Hide mouse cursor when over applet", 
					 &over_settings.cursor_off,
					 &over_settings.cursor_off};
static GaiCheckButton over_fade_between = {"Fade when mouse cursor is over",
					   &over_settings.fade,
					   &over_settings.fade};
static GaiRadioButton over_radio1 = {"Nothing", &over_settings.off, &over_settings.off, OVER_RADIO};
static GaiRadioButton over_radio2 = {"Matrix scroller", &over_settings.matrix, &over_settings.matrix, OVER_RADIO};
static GaiRadioButton over_radio3 = {"Plasma", &over_settings.plasma, &over_settings.plasma, OVER_RADIO};
static GaiRadioButton over_radio4 = {"Tetris", &over_settings.tetris, &over_settings.tetris, OVER_RADIO};


static GaiBox over_spec_box[] = {{GAI_RADIOBUTTON, 	&over_radio1},
				 {GAI_RADIOBUTTON, 	&over_radio2},
				 {GAI_RADIOBUTTON, 	&over_radio3},
				 {GAI_RADIOBUTTON, 	&over_radio4},
				 {GAI_END,		NULL}};
static GaiFrame over_frame = {"Mouse pointer over activates", 
			      NULL, 
			      (void *)&over_spec_box, 
			      NULL};


static GaiCheckButton xmms_on = {"On", &xmms_settings.on,&xmms_settings.on};

static GaiOptionMenu xmms_draw_om = {"Draw on:", 
				     draw_list, 
				     &xmms_settings.draw, 
				     &xmms_settings.draw,
				     XMMS_DRAW_OM};

static GaiOptionMenu xmms_vert_om = {"Vertical placement:",
				     vert_list, 
				     &xmms_settings.vert,
				     &xmms_settings.vert,
				     XMMS_VERT_OM};

static GaiOptionMenu xmms_horz_om = {"Horizontal placement:",
				     horz_list, 
				     &xmms_settings.horz,
				     &xmms_settings.horz,
				     XMMS_HORZ_OM};
static GaiOptionMenu xmms_scroll = {"Text scroll direction:",
				    xmms_scroll_list,
				    &xmms_settings.direction,
				    &xmms_settings.direction,
				    XMMS_SCROLL_OM};


static GaiColourSelector xmms_colour = {"Font colour",
					&xmms_settings.r,
					&xmms_settings.g,
					&xmms_settings.b,
					&xmms_settings.alpha,
					XMMS_FONT_COLOUR};


static GaiBox xmms_box[] ={{GAI_CHECKBUTTON,	&xmms_on},
			   {GAI_OPTIONMENU,	&xmms_draw_om},
			   {GAI_OPTIONMENU,	&xmms_vert_om},
			   {GAI_OPTIONMENU,	&xmms_horz_om},
			   {GAI_COLOURSELECTOR,	&xmms_colour},
			   {GAI_OPTIONMENU,	&xmms_scroll},
			   {GAI_END,		NULL}};

static GaiFrame xmms_frame = {"Xmms song name displayer",NULL, (void *)&xmms_box,NULL};

static GaiBox extra_box[] = {{GAI_CHECKBUTTON, 	&over_cursor_off},
			     {GAI_CHECKBUTTON, 	&over_fade_between},
			     {GAI_FRAME,	&over_frame},
			     {GAI_FRAME,	&xmms_frame},
			     {GAI_END,		NULL}};

/*--------------------------------------------------------------------------------*/
/* Leds */
/*--------------------------------------------------------------------------------*/
static char *leds_colour_list[] = {"Violet", "Dark blue", "Blue", "Light blue", "Blue-green", "Green",
				  "Yellow-green", "Yellow", "Orange", "Red", "Pink", NULL};
static char *leds_function_list[] = {"Off", "Numlock", "Capslock", "Scrollock", NULL};
static char *leds_vert_horz_list[] = {"Horizontally", "Vertically",NULL};
static char *leds_layout_list[] = {"All leds together", "Average distance", NULL};

static GaiOptionMenu led1_colour_om = {"Colour:", 
				       leds_colour_list,
				       &leds_settings.led1_colour, 
				       &leds_settings.led1_colour,
				       LEDS1_COLOUR_OM};
static GaiOptionMenu led2_colour_om = {"Colour:", 
				       leds_colour_list,
				       &leds_settings.led2_colour, 
				       &leds_settings.led2_colour,
				       LEDS2_COLOUR_OM};
static GaiOptionMenu led3_colour_om = {"Colour:", 
				       leds_colour_list,
				       &leds_settings.led3_colour, 
				       &leds_settings.led3_colour,
				       LEDS3_COLOUR_OM};
static GaiOptionMenu led4_colour_om = {"Colour:", 
				       leds_colour_list,
				       &leds_settings.led4_colour, 
				       &leds_settings.led4_colour,
				       LEDS4_COLOUR_OM};



static GaiOptionMenu led1_func_om = {"Function:", 
				     leds_function_list,
				     &leds_settings.led1_func, 
				     &leds_settings.led1_func,
				     LEDS1_FUNC_OM};
static GaiOptionMenu led2_func_om = {"Function:", 
				     leds_function_list,
				     &leds_settings.led2_func, 
				     &leds_settings.led2_func,
				     LEDS2_FUNC_OM};
static GaiOptionMenu led3_func_om = {"Function:", 
				     leds_function_list,
				     &leds_settings.led3_func, 
				     &leds_settings.led3_func,
				     LEDS3_FUNC_OM};

static GaiOptionMenu led4_func_om = {"Function:", 
				     leds_function_list,
				     &leds_settings.led4_func, 
				     &leds_settings.led4_func,
				     LEDS4_FUNC_OM};



static GaiOptionMenu leds_vert_om = {"Vertical placement:",
				     vert_list,
				     &leds_settings.vert,
				     &leds_settings.vert,
				     LEDS_VERT_OM};

static GaiOptionMenu leds_horz_om = {"Horizontal placement:",
				     horz_list,
				     &leds_settings.horz,
				     &leds_settings.horz,
				     LEDS_HORZ_OM};
static GaiOptionMenu leds_draw_om = {"Draw on:", 
				     draw_list,
				     &leds_settings.draw,
				     &leds_settings.draw,
				     LEDS_DRAW_OM};
static GaiOptionMenu leds_vert_horz_om = {"Draw the leds:",
					  leds_vert_horz_list,
					  &leds_settings.vert_horz,
					  &leds_settings.vert_horz,
					  LEDS_VERT_HORZ_OM};

static GaiOptionMenu leds_layout_om = {"Leds layout:",
				       leds_layout_list,
				       &leds_settings.layout,
				       &leds_settings.layout,
				       LEDS_LAYOUT_OM};



static GaiBox led1_box[] = {{GAI_OPTIONMENU, &led1_func_om},
			    {GAI_OPTIONMENU, &led1_colour_om},
			    {GAI_END, 	NULL}};
static GaiBox led2_box[] = {{GAI_OPTIONMENU, &led2_func_om},
			    {GAI_OPTIONMENU, &led2_colour_om},
			    {GAI_END, 	NULL}};
static GaiBox led3_box[] = {{GAI_OPTIONMENU, &led3_func_om},
			    {GAI_OPTIONMENU, &led3_colour_om},
			    {GAI_END, 	NULL}};
static GaiBox led4_box[] = {{GAI_OPTIONMENU, &led4_func_om},
			    {GAI_OPTIONMENU, &led4_colour_om},
			    {GAI_END, 	NULL}};


static GaiFrame led_frame1 = {"Led 1", "Led 2",
			      (void *)&led1_box, (void *)&led2_box};
static GaiFrame led_frame2 = {"Led 3", "Led 4",
			      (void *)&led3_box, (void *)&led4_box};




static GaiBox leds_box[] = {{GAI_OPTIONMENU,	&leds_vert_om},
			    {GAI_OPTIONMENU,	&leds_horz_om},
			    {GAI_OPTIONMENU,	&leds_vert_horz_om},
			    {GAI_OPTIONMENU,	&leds_layout_om},
			    {GAI_OPTIONMENU,	&leds_draw_om},
			    {GAI_FRAME, 	&led_frame1},
			    {GAI_FRAME, 	&led_frame2},
			    {GAI_END, 		NULL}};

/*--------------------------------------------------------------------------------*/
/* Thermometer */
/*--------------------------------------------------------------------------------*/

static char *messure_list[] = {
                               "Off",
#ifdef STATUS_HAVE_FAN1
			       "Fan 1",
#endif 
#ifdef STATUS_HAVE_FAN2
			       "Fan 2",
#endif 
#ifdef STATUS_HAVE_TEMP1
			       "Thermometer 1 (CPU)",
#endif 
#ifdef STATUS_HAVE_TEMP2
			       "Thermometer 2 (Motherboard)",
#endif 
#ifdef STATUS_HAVE_SWAP
			       "Swap usage",
#endif 
#ifdef STATUS_HAVE_DISC
			       "Disc usage",
#endif 
#ifdef STATUS_HAVE_CPU
			       "CPU load",
#endif 
#ifdef STATUS_HAVE_MEM
			       "Memory usage",
#endif 
#ifdef STATUS_HAVE_NET_ETH0
			       "Network ETH0 (1st network card) recieve",
			       "Network ETH0 (1st network card) send",
			       "Network ETH0 (1st network card) mixed",
#endif 
#ifdef STATUS_HAVE_NET_ETH1
			       "Network ETH1 (2nd network card) recieve",
			       "Network ETH1 (2nd network card) send",
			       "Network ETH1 (2nd network card) mixed",

#endif 
#ifdef STATUS_HAVE_NET_PPP0
			       "Network: PPP0 (Dial-up modem) recieve",
			       "Network: PPP0 (Dial-up modem) send",
			       "Network: PPP0 (Dial-up modem) mixed",
#endif 
#ifdef STATUS_HAVE_NET_LO
			       "Network: Loopback recieve",
			       "Network: Loopback send",
			       "Network: Loopback mixed",
#endif
			       NULL}; 

static GaiOptionMenu thermo1_draw_om = {"Draw on: ", 
				       draw_list, 
				       &thermometer_settings.draw1, 
				       &thermometer_settings.draw1,
				       THERMO_DRAW1_OM};

static GaiOptionMenu thermo1_vert_om = {"Vertical placement: ",
				       vert_list, 
				       &thermometer_settings.vert1,
				       &thermometer_settings.vert1,
				       THERMO_VERT1_OM};

static GaiOptionMenu thermo1_horz_om = {"Horizontal placement: ",
				       horz_list, 
				       &thermometer_settings.horz1,
				       &thermometer_settings.horz1,
				       THERMO_HORZ1_OM};

static GaiCheckButton thermo1_split = {"Split", &thermometer_settings.split1,&thermometer_settings.split1};

static GaiColourSelector thermo1_colour_sel = {"Thermometer colour: ",
					       &thermometer_settings.r1,
					       &thermometer_settings.g1,
					       &thermometer_settings.b1,
					       &thermometer_settings.alpha1,
					       THERMO1_COLOUR};

static GaiColourSelector thermo1_colour_s_sel = {"Thermometer colour: ",
					       &thermometer_settings.r1_s,
					       &thermometer_settings.g1_s,
					       &thermometer_settings.b1_s,
					       &thermometer_settings.alpha1_s,
					       THERMO1_S_COLOUR};

static GaiOptionMenu thermo1_messure_om = {"Messures: ",
					   messure_list, 
					   &thermometer_settings.messure1,
					   &thermometer_settings.messure1,
					   THERMO_MESSURE1_OM};

static GaiOptionMenu thermo1_messure_s_om = {"Messures: ",
					     messure_list, 
					     &thermometer_settings.messure1_s,
					     &thermometer_settings.messure1_s,
					     THERMO_MESSURE1_S_OM};



static GaiSpinButton thermo1_roof_s = {"Thermometer roof level: ",
				       &thermometer_settings.roof1_s,
				       &const_0,
				       &const_100000000,
				       &const_1,
				       &thermometer_settings.roof1_s};

static GaiSpinButton thermo1_roof = {"Thermometer roof level: ",
				     &thermometer_settings.roof1,
				     &const_0,
				     &const_100000000,
				     &const_1,
				     &thermometer_settings.roof1};

static GaiText thermo1_text1 = {"Fans, thermometer and network/modem needs a roof level limit set."};

static GaiTextEntry thermo1_mount_point = {"Mount point: ",
					   (void *)&thermometer_settings.mount_point1,
					   (void *)&thermometer_settings.mount_point1};
static GaiTextEntry thermo1_mount_point_s = {"Mount point: ",
					   (void *)&thermometer_settings.mount_point1_s,
					   (void *)&thermometer_settings.mount_point1_s};

static GaiText thermo1_text2 = {"A mount point is needed for disc usage."};


static GaiBox thermo1_split_box[] = {{GAI_COLOURSELECTOR,	(void *)&thermo1_colour_s_sel},
				     {GAI_OPTIONMENU,		(void *)&thermo1_messure_s_om},
				     {GAI_SPINBUTTON,		(void *)&thermo1_roof_s},
				     {GAI_TEXT,			(void *)&thermo1_text1},
				     {GAI_TEXTENTRY,		(void *)&thermo1_mount_point_s},
				     {GAI_TEXT,			(void *)&thermo1_text2},
				     {GAI_END,			NULL}};

static GaiFrame thermo1_split_frame = {"Split", NULL, (void *)&thermo1_split_box,NULL};

static GaiBox thermo1_box[] = {{GAI_OPTIONMENU,		(void *)&thermo1_draw_om},
			       {GAI_OPTIONMENU,		(void *)&thermo1_vert_om},
			       {GAI_OPTIONMENU,		(void *)&thermo1_horz_om},
			       			       {GAI_CHECKBUTTON,	(void *)&thermo1_split},
			       {GAI_COLOURSELECTOR,	(void *)&thermo1_colour_sel},
			       {GAI_OPTIONMENU,		(void *)&thermo1_messure_om},
			       {GAI_SPINBUTTON,		(void *)&thermo1_roof},
			       {GAI_TEXT,		(void *)&thermo1_text1},
			       {GAI_TEXTENTRY,		(void *)&thermo1_mount_point},
			       {GAI_TEXT,		(void *)&thermo1_text2},
			       {GAI_FRAME,		(void *)&thermo1_split_frame},
			       {GAI_END,		NULL}};




static GaiOptionMenu thermo2_draw_om = {"Draw on: ", 
				       draw_list, 
				       &thermometer_settings.draw2, 
				       &thermometer_settings.draw2,
				       THERMO_DRAW2_OM};

static GaiOptionMenu thermo2_vert_om = {"Vertical placement: ",
				       vert_list, 
				       &thermometer_settings.vert2,
				       &thermometer_settings.vert2,
				       THERMO_VERT2_OM};

static GaiOptionMenu thermo2_horz_om = {"Horizontal placement: ",
				       horz_list, 
				       &thermometer_settings.horz2,
				       &thermometer_settings.horz2,
				       THERMO_HORZ2_OM};

static GaiCheckButton thermo2_split = {"Split", &thermometer_settings.split2,&thermometer_settings.split2};

static GaiColourSelector thermo2_colour_sel = {"Thermometer colour: ",
					       &thermometer_settings.r2,
					       &thermometer_settings.g2,
					       &thermometer_settings.b2,
					       &thermometer_settings.alpha2,
					       THERMO2_COLOUR};

static GaiColourSelector thermo2_colour_s_sel = {"Thermometer colour: ",
					       &thermometer_settings.r2_s,
					       &thermometer_settings.g2_s,
					       &thermometer_settings.b2_s,
					       &thermometer_settings.alpha2_s,
					       THERMO2_S_COLOUR};

static GaiOptionMenu thermo2_messure_om = {"Messures: ",
					   messure_list, 
					   &thermometer_settings.messure2,
					   &thermometer_settings.messure2,
					   THERMO_MESSURE2_OM};

static GaiOptionMenu thermo2_messure_s_om = {"Messures: ",
					     messure_list, 
					     &thermometer_settings.messure2_s,
					     &thermometer_settings.messure2_s,
					     THERMO_MESSURE2_S_OM};
static GaiSpinButton thermo2_roof_s = {"Thermometer roof level: ",
				       &thermometer_settings.roof2_s,
				       &const_0,
				       &const_100000000,
				       &const_1,
				       &thermometer_settings.roof2_s};

static GaiSpinButton thermo2_roof = {"Thermometer roof level: ",
				     &thermometer_settings.roof2,
				     &const_0,
				     &const_100000000,
				     &const_1,
				     &thermometer_settings.roof2};



static GaiTextEntry thermo2_mount_point = {"Mount point: ",
					   (void *)&thermometer_settings.mount_point2,
					   (void *)&thermometer_settings.mount_point2};
static GaiTextEntry thermo2_mount_point_s = {"Mount point: ",
					   (void *)&thermometer_settings.mount_point2_s,
					   (void *)&thermometer_settings.mount_point2_s};


static GaiBox thermo2_split_box[] = {{GAI_COLOURSELECTOR,	(void *)&thermo2_colour_s_sel},
				     {GAI_OPTIONMENU,		(void *)&thermo2_messure_s_om},
				     {GAI_SPINBUTTON,		(void *)&thermo2_roof_s},
				     {GAI_TEXT,			(void *)&thermo1_text1},
				     {GAI_TEXTENTRY,		(void *)&thermo2_mount_point_s},
				     {GAI_TEXT,			(void *)&thermo1_text2},
				     {GAI_END,			NULL}};

static GaiFrame thermo2_split_frame = {"Split", NULL, (void *)&thermo2_split_box,NULL};

static GaiBox thermo2_box[] = {{GAI_OPTIONMENU,		(void *)&thermo2_draw_om},
			       {GAI_OPTIONMENU,		(void *)&thermo2_vert_om},
			       {GAI_OPTIONMENU,		(void *)&thermo2_horz_om},
			       {GAI_CHECKBUTTON,	(void *)&thermo2_split},
			       {GAI_COLOURSELECTOR,	(void *)&thermo2_colour_sel},
			       {GAI_OPTIONMENU,		(void *)&thermo2_messure_om},
			       {GAI_SPINBUTTON,		(void *)&thermo2_roof},
			       {GAI_TEXT,		(void *)&thermo1_text1},
			       {GAI_TEXTENTRY,		(void *)&thermo2_mount_point},
			       {GAI_TEXT,		(void *)&thermo1_text2},
			       {GAI_FRAME,		(void *)&thermo2_split_frame},
			       {GAI_END,		NULL}};



static GaiFrame thermo_frame = {"First Thermometer", "Second Thermometer", (void *)&thermo1_box,(void *)&thermo2_box};
static GaiBox thermo_main_box[] = {{GAI_FRAME,		(void *)&thermo_frame},
				   {GAI_END,		NULL}};


/*--------------------------------------------------------------------------------*/
GaiNoteBook shermans_pref[] = {{"General", 	(void *)&box},
			       {"Fish",		(void *)&main_fish_box},
			       {"Time", 	(void *)&clock_date_box},

			       //{"Leds",		(void *)&leds_box},
			       {"Thermometer", 	(void *)&thermo_main_box},
			       {"Extra",	(void *)&extra_box},
			       {NULL,NULL}};



void pref_restart(gpointer d)
{
    Background_settings *bas;
    Bottom_settings *bos;
    Bubble_settings *bus;
    Clock_settings *cs;
    Date_settings *ds;
    Fish_settings *fs;
    General_settings *gs;
    Leds_settings *ls;
    Over_settings *os;
    Sound_settings *ss;
    Thermometer_settings *ts;
    Xmms_sn_settings *xss;
    AquariumData *ad;

    //printf("pref restart\n");
    
    ad = aquarium_get_settings_ptr();
    ad->dont_update = 1;
    
    bas = background_get_settings_ptr();
    bos = bottom_get_settings_ptr();
    bus = bubble_get_settings_ptr();
    cs = clock_get_settings_ptr();
    ds = date_get_settings_ptr();
    fs = fish_get_settings_ptr();
    gs = general_get_settings_ptr();

    /*
      ls = leds_get_settings_ptr();*/
    os = over_get_settings_ptr();
    ss = sound_get_settings_ptr();
    ts = thermometer_get_settings_ptr();
    xss = xmms_sn_get_settings_ptr();


    memcpy(bas, &background_settings, sizeof(Background_settings));
    memcpy(bos, &bottom_settings, sizeof(Bottom_settings));
    memcpy(bus, &bubble_settings, sizeof(Bubble_settings));
    memcpy(cs, &clock_settings, sizeof(Clock_settings));
    memcpy(ds, &date_settings, sizeof(Date_settings));
    memcpy(fs, &fish_settings, sizeof(Fish_settings));
    memcpy(gs, &general_settings, sizeof(General_settings));

    /*
      memcpy(ls, &leds_settings, sizeof(Leds_settings));*/

    memcpy(os, &over_settings, sizeof(Over_settings));
    memcpy(ss, &sound_settings, sizeof(Sound_settings));

    
    memcpy(ts, &thermometer_settings, sizeof(Thermometer_settings)); 
    memcpy(xss, &xmms_settings, sizeof(Xmms_sn_settings));


    //printf("Save settings\nclock\n");
    clock_save_settings();
    //printf("date\n");
    date_save_settings();
    //printf("buble\n");
    bubble_save_settings();
    //printf("background\n");
    background_save_settings();
    //printf("fish settings\n");
    fish_save_settings();
    //printf("General settings\n");
    general_save_settings();
    //printf("bottom save settings\n");
    bottom_save_settings();

    //printf("over save settings\n");
    over_save_settings();
    //printf("leds save settings\n");
    //    leds_save_settings();
    thermometer_save_settings();
    //printf("xmms save settings\n");
    xmms_sn_save_settings();
    //printf("sound save settings\n");
    sound_save_settings();

    aquarium_free_all();
    aquarium_reload_all();
    ad->dont_update = 0;

    if(gai_applet_mode() == GAI_DOCKAPP){
	if(gs->ratio_width > gs->ratio_height)
	    aquarium_change(GAI_HORIZONTAL, gai_get_size(), gai_get_size(), NULL);
	else
	    aquarium_change(GAI_VERTICAL, gai_get_size(), gai_get_size(), NULL);
    }
    else
	aquarium_change(gai_get_orient(), gai_get_size(), gai_get_size(), NULL);
}

void pref_init(void)
{
    Background_settings *bas;
    Bottom_settings *bos;
    Bubble_settings *bus;
    Clock_settings *cs;
    Date_settings *ds;
    Fish_settings *fs;
    Leds_settings *ls;
    Over_settings *os;
    Sound_settings *ss;
    Thermometer_settings *ts;
    Xmms_sn_settings *xss;
    General_settings *gs;

    AquariumData *ad;

    
    ad = aquarium_get_settings_ptr();
    
    bas = background_get_settings_ptr();
    bos = bottom_get_settings_ptr();
    bus = bubble_get_settings_ptr();
    cs = clock_get_settings_ptr();
    ds = date_get_settings_ptr();
    fs = fish_get_settings_ptr();
    gs = general_get_settings_ptr();

    /* ls = leds_get_settings_ptr(); */
    os = over_get_settings_ptr();
    ss = sound_get_settings_ptr();
    ts = thermometer_get_settings_ptr();
    xss = xmms_sn_get_settings_ptr();
   

    memcpy(&background_settings, bas, sizeof(Background_settings));
    memcpy(&bottom_settings, bos, sizeof(Bottom_settings));
    memcpy(&bubble_settings, bus, sizeof(Bubble_settings));
    memcpy(&clock_settings, cs, sizeof(Clock_settings));
    memcpy(&date_settings, ds, sizeof(Date_settings));
    memcpy(&fish_settings, fs, sizeof(Fish_settings));
    memcpy(&general_settings, gs, sizeof(General_settings));

    /* memcpy(&leds_settings, ls, sizeof(Leds_settings));*/

    memcpy(&over_settings, os, sizeof(Over_settings));
    memcpy(&sound_settings, ss, sizeof(Sound_settings));
    memcpy(&thermometer_settings, ts, sizeof(Thermometer_settings));
    memcpy(&xmms_settings, xss, sizeof(Xmms_sn_settings));

}
